"use strict";
/**
 * @module frest-json
 */
/**
 *    Copyright 2018 Panjie Setiawan Wicaksono
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var ids_1 = require("./ids");
/**
 * Create a JSON response interceptor.
 *
 * @remarks
 * This interceptor will try to parse response's body if the response's
 * `Content-Type` header is compatible with the one provided in
 * {@link IJSONResponseInterceptorOption.headerContent} option.
 * If {@link IJSONResponseInterceptorOption.transform} is provided, it'll
 * call the function before parsing it.
 *
 * @param opts - Options for this response interceptor
 *
 * @returns JSON response interceptor
 */
function responseInterceptor(opts) {
    if (opts === void 0) { opts = {}; }
    var jsonResponseInterceptor = function (input) {
        return new Promise(function (resolve, reject) {
            var force = opts.force, headerContent = opts.headerContent, transform = opts.transform;
            var skip = input.request.skip;
            var _a = input.response, origin = _a.origin, originBody = _a.body;
            var headers = origin.headers, bodyUsed = origin.bodyUsed, status = origin.status;
            var ct = headers.get('Content-Type');
            if (!bodyUsed &&
                (!skip || skip.indexOf(ids_1.ID_RESPONSE) < 0) &&
                ((ct && ct.indexOf(headerContent || 'application/json') >= 0) || force)) {
                var promise = void 0;
                if (transform) {
                    promise = transform(origin)
                        .then(function (text) { return JSON.parse(text); })
                        .then(function (body) {
                        resolve({ origin: origin, body: body });
                    });
                }
                else {
                    promise = origin.json().then(function (body) {
                        resolve({ origin: origin, body: body });
                    });
                }
                // If the HTTP status is 201-204, assume it's empty intentionally
                // and just resolve the origin, or reject it otherwise.
                promise.catch(function (err) {
                    if (status >= 201 && status <= 204) {
                        resolve({ origin: origin });
                    }
                    else {
                        reject(err);
                    }
                });
                return;
            }
            resolve({ origin: origin, body: originBody });
        });
    };
    Object.defineProperty(jsonResponseInterceptor, 'id', { value: ids_1.ID_RESPONSE });
    return jsonResponseInterceptor;
}
exports.default = responseInterceptor;
